﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using C1.WPF;
using C1.WPF.Theming.ExpressionDark;
using C1.WPF.Theming.C1Blue;
using C1.WPF.Theming.Cosmopolitan;
using C1.WPF.Theming.ExpressionLight;
using C1.WPF.FlexGrid;

namespace Wpf_C1TreeView_Grid
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        ObservableCollection<Employee> emp;
        C1ThemeExpressionLight theme;
        C1DragDropManager dd, ddm;
        C1TreeViewItem treeItem;
        int row, col;
        bool flag;

        public MainWindow()
        {
            InitializeComponent();
            emp = new ObservableCollection<Employee>();
            populateData();
            theme = new C1ThemeExpressionLight();
            dd = new C1DragDropManager();
            ddm = new C1DragDropManager();
            register();
            c1FlexGrid.CellFactory = new DragDropCellFactory(ddm);
                        
            dd.DragStart += (s, e) =>
            {
                try
                {
                    treeItem = e.DragSource as C1TreeViewItem;
                }
                catch (Exception ee)
                {
                    MessageBox.Show("Can not drag this element! Please try another one.");
                }
            };
            
            dd.DragDrop += (s, e) =>
            {
                try
                {
                    var grid = e.DropTarget as C1FlexGrid;
                    if (grid.Name == "c1FlexGrid")
                    {
                        if (flag)
                        {
                            grid[row, col] = treeItem.Header.ToString();
                            flag = false;
                        }
                        else
                        {
                            if (treeItem.Items.Count > 1)
                            {
                                emp.Add(new Employee()
                                {
                                    ID = (treeItem.Items[0] as C1TreeViewItem).Header.ToString(),
                                    Name = (treeItem.Items[1] as C1TreeViewItem).Header.ToString(),
                                    Department = (treeItem.Items[2] as C1TreeViewItem).Header.ToString(),
                                    Location = (treeItem.Items[3] as C1TreeViewItem).Header.ToString()
                                });
                            }
                            else
                                emp.Add(new Employee()
                                {
                                    ID = treeItem.Header.ToString(),
                                    Name = treeItem.Header.ToString(),
                                    Department = treeItem.Header.ToString(),
                                    Location = treeItem.Header.ToString()
                                });
                        }
                    }
                }
                catch (Exception ee)
                {
                    MessageBox.Show("Can not drop this element! Please try another one.");
                }
            };      
            
            ddm.DragStart += (s, e) =>
            {
                try
                {
                    var bdr = e.DragSource as Border;
                    var row = (bdr.Tag as Row);
                    var flex = (bdr.Tag as Row).Grid;
                    ddm.TargetMarker.Width = flex.ActualWidth;  //configure the Target Marker
                }
                catch (Exception ee)
                {
                    MessageBox.Show("Can not drag this element! Please try another one.");
                }
            };

            ddm.DragDrop += (s, e) =>
            {
                try
                {
                    var bdr = e.DragSource as Border;
                    var treeview = e.DropTarget as C1TreeViewItem;
                    if (treeview.Name == "c1TreeViewItem")  //check the actual drop target
                    {
                        int index = c1FlexGrid.Selection.Row;
                        C1TreeViewItem item = new C1TreeViewItem();
                        item.Header = c1FlexGrid[index, 0];
                        for (int i = 0; i < c1FlexGrid.Columns.Count; i++)
                        {
                            C1TreeViewItem subitem = new C1TreeViewItem();
                            subitem.Header = c1FlexGrid[index, i];
                            dd.RegisterDragSource(subitem, DragDropEffect.Copy, ModifierKeys.None, true);
                            item.Items.Add(subitem);
                        }
                        dd.RegisterDragSource(item, DragDropEffect.Copy, ModifierKeys.None, true);
                        treeview.Items.Add(item);
                    }
                }
                catch (Exception ee)
                {
                    MessageBox.Show("Can not drop this element! Please try another one.");
                }
            };

            theme.Apply(this);
            tb.Width = myPanel.Width;
        }

        void c1FlexGrid_PrepareCellForEdit(object sender, CellEditEventArgs e)
        {
            row = e.Row;
            col = e.Column;
        }

        void register()
        {
            dd.RegisterDragSource(c1TreeView, DragDropEffect.Copy, ModifierKeys.None, true);
            dd.RegisterDragSource(c1TreeViewItem, DragDropEffect.Copy, ModifierKeys.None, true);
            dd.RegisterDragSource(emp1, DragDropEffect.Copy, ModifierKeys.None, true);
            dd.RegisterDragSource(e1, DragDropEffect.Copy, ModifierKeys.None, true);
            dd.RegisterDragSource(pr, DragDropEffect.Copy, ModifierKeys.None, true);
            dd.RegisterDragSource(dt, DragDropEffect.Copy, ModifierKeys.None, true);
            dd.RegisterDragSource(ind, DragDropEffect.Copy, ModifierKeys.None, true);
            dd.RegisterDragSource(emp2, DragDropEffect.Copy, ModifierKeys.None, true);
            dd.RegisterDragSource(e2, DragDropEffect.Copy, ModifierKeys.None, true);
            dd.RegisterDragSource(dn, DragDropEffect.Copy, ModifierKeys.None, true);
            dd.RegisterDragSource(dv, DragDropEffect.Copy, ModifierKeys.None, true);
            dd.RegisterDragSource(us, DragDropEffect.Copy, ModifierKeys.None, true);
            dd.RegisterDropTarget(c1FlexGrid, true);

            ddm.RegisterDropTarget(c1TreeViewItem, true);
        }

        void populateData()
        {
            emp.Add(new Employee() { ID = "101", Name = "John", Department = "Dev Tools", Location = "US" });
            emp.Add(new Employee() { ID = "102", Name = "Mike", Department = "Development", Location = "UK" });
            emp.Add(new Employee() { ID = "103", Name = "Fuller", Department = "R&D", Location = "US" });
            emp.Add(new Employee() { ID = "104", Name = "Lilly", Department = "HR", Location = "Japan" });
            emp.Add(new Employee() { ID = "105", Name = "Prabhat", Department = "Accounts", Location = "India" });
            emp.Add(new Employee() { ID = "106", Name = "Richa", Department = "HR", Location = "India" });
            emp.Add(new Employee() { ID = "107", Name = "Miller", Department = "Dev Tools", Location = "US" });
            emp.Add(new Employee() { ID = "108", Name = "Sam", Department = "Dev Tools", Location = "UK" });
            this.DataContext = emp;
        }

        private void c1FlexGrid_BeginningEdit(object sender, CellEditEventArgs e)
        {
            row = e.Row;
            col = e.Column;
            flag = true;
        }
    } 
}
